from .console import console


# spinners
def spinner(*args, **kwargs):
    return console.status(*args, spinner="line", **kwargs)


def loading(*args, **kwargs):
    return console.status(*args, spinner="aesthetic", **kwargs)


def clock(*args, **kwargs):
    return console.status(*args, spinner="clock", **kwargs)



from rich.console import Console
from .colors import THEME

console = Console(theme=THEME)

from rich.traceback import install

install(show_locals=True)


def console_print(*args, **kwargs):

    from .wrappers import CURRENT_PROGRESS, clear_progress

    if CURRENT_PROGRESS is not None:
        if CURRENT_PROGRESS.finished:
            clear_progress()
            return console.print(*args, **kwargs)
        else:
            return CURRENT_PROGRESS.console.print(*args, **kwargs)

    return console.print(*args, **kwargs)



COLOR_LOOKUP = {
    "var_name": "cyan",
    "var_type": "hot_pink",
    "file": "gold3",
    "result": "cornflower_blue",
    "success": "bold green",
    "error": "bold red",
    "warning": "dark_orange",
    "arg": "cyan3",
    "debug": "bright_black",
}

from rich.theme import Theme

THEME = Theme(COLOR_LOOKUP, inherit=True)



def df_to_table(df):

    from rich.table import Table
    import pandas as pd

    table = Table()

    if isinstance(df.index, pd.MultiIndex):
        multi = True
        for name in df.index.names:
            table.add_column(name)
    else:
        multi = False
        if df.index.name:
            table.add_column(df.index.name)
        else:
            table.add_column("index")

    for col in df.columns:
        table.add_column(col)

    for i, row in df.iterrows():

        values = []

        if multi:
            for name in i:
                values.append(str(name))
        else:
            values.append(str(i))

        for col in df.columns:
            values.append(str(row[col]))
        table.add_row(*values)

    return table



from .console import console, console_print
from rich.text import Text
from .colors import COLOR_LOOKUP
from .tools import strip_formats, restyle_arg

### STYLES


def bold(*messages, **kwargs):
    text, formats = strip_formats(*messages, **kwargs)
    text = Text(text.strip())
    text.stylize("bold")
    for style, start, end in formats:
        text.stylize(style, start, end)
    return console_print(text)


def italic(*messages, **kwargs):
    text, formats = strip_formats(*messages, **kwargs)
    text = Text(text.strip())
    text.stylize("italic")
    for style, start, end in formats:
        text.stylize(style, start, end)
    return console_print(text)


def underline(*messages, **kwargs):
    text, formats = strip_formats(*messages, **kwargs)
    text = Text(text.strip())
    text.stylize("underline")
    for style, start, end in formats:
        text.stylize(style, start, end)
    return console_print(text)


### LOG


def warning(*messages, **kwargs):
    text = " Warning "
    text, formats = strip_formats(*messages, text=text, **kwargs)
    text = Text(f"{text}!")
    text.stylize("warning")
    text.stylize("reverse bold", 0, 9)
    for style, start, end in formats:
        text.stylize(style, start, end)
    return console_print(text)


def error(*messages, **kwargs):
    text = " ERROR "
    text, formats = strip_formats(*messages, text=text, **kwargs)
    text = Text(f"{text}!")
    text.stylize("error")
    text.stylize("reverse", 0, 7)
    for style, start, end in formats:
        text.stylize(style, start, end)
    return console_print(text)


def success(*messages, **kwargs):
    text = " Success "
    text, formats = strip_formats(*messages, text=text, **kwargs)
    text = Text(f"{text}!")
    text.stylize("success")
    text.stylize("reverse", 0, 9)
    for style, start, end in formats:
        text.stylize(style, start, end)
    return console_print(text)


def debug(*messages, **kwargs):
    text = "DEBUG:"
    text, formats = strip_formats(*messages, text=text, **kwargs)
    text = Text(f"{text}")
    text.stylize("debug")
    for style, start, end in formats:
        text.stylize(style, start, end)
    return console_print(text)


def prompt(*messages, **kwargs):
    text = ">>>"
    text, formats = strip_formats(*messages, text=text, **kwargs)
    text = Text(f"{text}")
    text.stylize("bold purple")
    for style, start, end in formats:
        text.stylize(style, start, end)
    return console_print(text)


def disk(message: str, *, prefix: str):
    message = str(message)
    text = Text(f" DISK  {prefix} {message}...")
    text.stylize("file", 0, 6)
    text.stylize("reverse bold", 0, 6)
    text.stylize("file", 8 + len(prefix), 8 + len(prefix) + len(message))
    return console_print(text)


def reading(message):
    return disk(message, prefix="Reading")


def writing(message):
    return disk(message, prefix="Writing")


def var(
    *args,
    separator: str = "=",
    color: str | None = None,
    highlight: bool = True,
    highlight_if_rich_dunder: bool = False,
):

    nargs = len(args)

    if nargs == 1:
        import re
        import inspect

        # get the variable name as defined in the code that called this function
        frame = inspect.currentframe().f_back
        call_line = inspect.getframeinfo(frame).code_context[0].strip()
        match = re.search(r"var\((.+)\)", call_line)

        if match:
            variable = match.group(1).strip()
        else:
            variable = "arg"

        value = args[0]
        unit = None
    elif nargs == 2:
        variable = args[0]
        value = args[1]
        unit = None
    elif nargs == 3:
        variable = args[0]
        value = args[1]
        unit = args[2]
    else:
        raise ValueError("Wrong number of arguments to mrich.var()")

    # style variable
    variable = Text(str(variable), style=COLOR_LOOKUP["var_name"])
    variable.stylize("bold")

    # to some type conversions if appropriate
    value = restyle_arg(value)

    if "Path" in str(type(value)):
        color = "file"

    if color and color in COLOR_LOOKUP:
        color = COLOR_LOOKUP[color]

    if hasattr(value, "__rich__"):
        highlight = highlight_if_rich_dunder

    if color:
        value = Text(str(value), style=color)

    objects = [variable, separator, value]

    if unit:
        unit = Text(unit, style=COLOR_LOOKUP["var_type"])
        objects.append(unit)

    console.print(*objects, markup=True, highlight=highlight)


### HEADINGS


def h1(message):
    from rich.panel import Panel

    text = Text(message.upper(), justify="center")
    text.stylize("bold")
    return console.print(Panel(text))


def h2(message):
    return console.rule(message)


def h3(message):
    from rich.panel import Panel

    return console.print(Panel.fit(message))


### ALIASES


def header(*args, **kwargs):
    return bold(*args, **kwargs)



import re

dict_keys = type(dict().keys())


def detect_format_prefix(message):

    match = re.match(r"^\[.*?\]", message)

    if match:
        match = match.group()

    return match


def strip_formats(*messages, text="", separator=" "):
    formats = []
    start = len(text) + 1
    for message in messages:

        if hasattr(message, "__rich__"):
            message = message.__rich__()
        else:
            message = str(message)

        prefix = detect_format_prefix(message)

        prefix_found = prefix is not None

        if prefix_found:
            message = message.removeprefix(prefix)
            prefix = prefix.removeprefix("[").removesuffix("]")

        end = start + len(message)
        text += separator + message

        if prefix_found:
            formats.append((prefix, start, end + 2))

        start = end

    return text, formats


def restyle_arg(arg):

    try:
        from pandas import DataFrame, Index

        if isinstance(arg, DataFrame):
            from .df import df_to_table

            # console_print(arg, type(arg))
            table = df_to_table(arg)
            return table

        elif isinstance(arg, Index):
            return set(arg)

    except ModuleNotFoundError:
        pass

    if isinstance(arg, dict_keys):
        return set(arg)

    else:
        return arg
